==> 0.3.19.tar.gz: android/src/main/java/com/samelement/iot_notification/service/AccessTokenAuthenticator.java <==
package com.samelement.iot_notification.service;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.samelement.iot_notification.screen.IntentKey;

import okhttp3.Authenticator;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.Route;

public class AccessTokenAuthenticator implements Authenticator {

    private final AccessTokenRepository accessTokenRepository;

    public AccessTokenAuthenticator(AccessTokenRepository accessTokenRepository) {
        this.accessTokenRepository = accessTokenRepository;
    }

    @Nullable
    @Override
    public Request authenticate(Route route, Response response) {
        // ambil token saat ini
        final String accessToken = accessTokenRepository.getUserAccessToken();

        if (!isRequestWithAccessToken(response) || accessToken == null) {
            return null;
        }

        synchronized (this) {
            final String newAccessToken = accessTokenRepository.getUserAccessToken();

            // Access token is refreshed in another thread.
            // cek tokennya wes berubah belum
            // kalau sudah pake token yang baru
            if (!accessToken.equals(newAccessToken)) {
                return response.request().newBuilder()
                        .removeHeader(IntentKey.KEY_AUTH)
                        .addHeader(IntentKey.KEY_AUTH, "Bearer "+newAccessToken)
                        .build();
            }

            // tokennya masih sama, jadi masih 401, refresh yuk
            // Need to refresh an access token
            final String updatedDevAccessToken = accessTokenRepository.refreshDevAccessToken();
            final String updatedUserAccessToken = accessTokenRepository.refreshUserAccessToken();

            // kalau sudah dapat refreshannya, retry request ini
            return response.request()
                    .newBuilder()
                    .removeHeader(IntentKey.KEY_AUTH)
                    .addHeader(IntentKey.KEY_AUTH, "Bearer "+updatedUserAccessToken)
                    .build();
        }
    }

    private boolean isRequestWithAccessToken(@NonNull Response response) {
        String header = response.request().header("Authorization");
        return header != null && header.startsWith("Bearer");
    }
}

==> 0.5.4.tar.gz: android/src/main/java/com/samelement/iot_notification/service/AccessTokenAuthenticator.java <==
package com.samelement.iot_notification.service;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.samelement.iot_notification.screen.IntentKey;

import okhttp3.Authenticator;
import okhttp3.Request;
import okhttp3.Response;
import okhttp3.Route;

public class AccessTokenAuthenticator implements Authenticator {

    private final AccessTokenRepository accessTokenRepository;

    public AccessTokenAuthenticator(AccessTokenRepository accessTokenRepository) {
        this.accessTokenRepository = accessTokenRepository;
    }

    @Nullable
    @Override
    public Request authenticate(Route route, @NonNull Response response) {
        // ambil token saat ini
        final String accessToken = accessTokenRepository.getUserAccessToken();

        if (!isRequestWithAccessToken(response) || accessToken == null) {
            return null;
        }

        synchronized (this) {
            final String newAccessToken = accessTokenRepository.getUserAccessToken();

            // Access token is refreshed in another thread.
            // cek tokennya wes berubah belum
            // kalau sudah pake token yang baru
            if (!accessToken.equals(newAccessToken)) {
                return response.request().newBuilder()
                        .removeHeader(IntentKey.KEY_AUTH)
                        .addHeader(IntentKey.KEY_AUTH, "Bearer "+newAccessToken)
                        .build();
            }

            // tokennya masih sama, jadi masih 401, refresh yuk
            // Need to refresh an access token
            final String updatedDevAccessToken = accessTokenRepository.refreshDevAccessToken();
            final String updatedUserAccessToken = accessTokenRepository.refreshUserAccessToken();

            // kalau sudah dapat refreshannya, retry request ini
            return response.request()
                    .newBuilder()
                    .removeHeader(IntentKey.KEY_AUTH)
                    .addHeader(IntentKey.KEY_AUTH, "Bearer "+updatedUserAccessToken)
                    .build();
        }
    }

    private boolean isRequestWithAccessToken(@NonNull Response response) {
        String header = response.request().header("Authorization");
        return header != null && header.startsWith("Bearer");
    }
}
